function [codemap,h,codes] = HEP_CLBP_MC(img, r, ftrType, crcType, fltType)
%Combination (cartesian product) of Completed Local Binary Patterns 'M' and
%'C'
%**********
%Synopsis:  HEP_CLBP_MC(img, r, ftrType, crcType, fltType)
%**********
%INPUT
%img:       The input image. An n x m matrix of double or integers.
%r:         The radius of the circle. An integer representing the resolution 
%           (in pixels) at which HEP_CLBP_MC is computed.
%ftrType:   The feature type. Can be:
%               -> 'dir'    (directional, non rotation-invariant features)
%               -> 'CN'     (rotation-invariant features, i.e.: cyclic group CN)
%               -> 'DN'     (rotation and reflection-invariant features, 
%                            i.e.: dihedral group DN)
%               -> 'CNu2'   (same as 'CN', but uniform patterns. Note that 
%                            these are also reflection-invariant patterns [2])
%           For deatils see Ref. [2].
%crcType:   The type of digital circle - for possible values see DrawCircle()
%fltType:   How filtering is performed across the image borders - for
%           possible values see CreateImageLayers()
%**********
%OUTPUT
%codemap:   The map of HEP_CLBP_MC codes (codeword image)
%h:         The normalised histogram of HEP_CLBP_MC codes (sums one)
%codes:     The possible codes
%**********
%
%   Sample usage
%   ------------
%      %Example 1
%      %Resolution = 2 pixels, directional features, circular repetition across
%      %the image borders, Andres' circles
%      img = imread('rice.png');
%      [codemap, h] = HEP_CLBP_MC(img, 2, 'dir', 'Andres', 'circular');
%
%      %Example 2
%      %Resolution = 1 pixels, rotation-invariant features, symmetric repetition across
%      %the image borders, Andres' circles
%      img = imread('rice.png');
%      [codemap, h] = HEP_CLBP_MC(img, 2, 'CN', 'Andres', 'symmetric');
%
%----------
%References
%[1]        Guo, Z., Zhang, L., Zhang, D.
%           A completed modeling of local binary pattern operator for 
%           texture classification (2010) IEEE Transactions on Image 
%           Processing, 19 (6), art. no. 5427137, pp. 1657-1663.
%[2]        Bianconi, F., Gonzlez, E.
%           Counting local n-ary patterns
%           (2019) Pattern Recognition Letters, 117, pp. 24-29. 

    [rows cols] = size(img);

    %Compute CLBP_C 
    [codemap_CLBP_C,~,codes_CLBP_C] = ...
        HEP_CLBP_C(img, r, ftrType, crcType, fltType);
    
    %Compute CLBP_M 
    [codemap_CLBP_M,~,codes_CLBP_M] = ...
        HEP_CLBP_M(img, r, ftrType, crcType, fltType);
    
    %Compute the codes generated by the cartesian product C x M
    codes_CLBP_C_repeated = repmat(codes_CLBP_C, [1 numel(codes_CLBP_M)]) + 1;
    codes_CLBP_C_repeated = reshape(codes_CLBP_C_repeated, [1 numel(codes_CLBP_C_repeated)]);
    codes_CLBP_M_repeated = repmat(codes_CLBP_M, [1 numel(codes_CLBP_C)]) + 1;
    codes_CLBP_MC = sub2ind([max(codes_CLBP_C) + 1, max(codes_CLBP_M) + 1],...
                            codes_CLBP_C_repeated,...
                            codes_CLBP_M_repeated);     
    codes = codes_CLBP_MC - 1;
    codes = sort(codes, 'ascend');
    
    %Do the same with the codemaps now
    codemap_CLBP_C = reshape(codemap_CLBP_C', [1 numel(codemap_CLBP_C)]) + 1;
    codemap_CLBP_M = reshape(codemap_CLBP_M', [1 numel(codemap_CLBP_M)]) + 1;
    codemap_CLBP_MC = sub2ind([max(codes_CLBP_C) + 1 max(codes_CLBP_M) + 1],...
                               codemap_CLBP_C,...
                               codemap_CLBP_M);
    codemap = codemap_CLBP_MC - 1;
    codemap = reshape(codemap, [rows cols]);
        
    %Compute the normalised histogram
    h = hist(codemap(:), codes)/numel(codemap);

end