function [invCodemap, invCodes] = PostprocessCodes(codemap, nBeads, nColours, tGroup, oType, nType, pType)
%Conversion of codes generated by histograms of equivalent patterns 
%into action-invariant codes. 
%**********
%Synopsis:      [invCodemap, invCodes] = PostprocessCodes(codemap, nBeads,...
%                   nColours, tGroup, oType, nType, pType)
%**********
%INPUT
%codemap:       A map of codes (M x N matrix of integers)
%nBeads:        Number of pixels in the periphery of the neighbourhood
%nColours:      Number of levels allowed for each pixel
%nType:         The type of neighbourhood. Can be:
%                   'FULL' -> periphery + central pixel
%                   'PERI' -> periphery only
%tGroup:        The transformation group under which the codes have to be 
%               invariant. Can be:
%                   'CN'   -> cyclic group of order nBeads (i.e.: invariance 
%                             under rotations)
%                   'DN'   -> dihedral group of nBeads (i.e.: invariance 
%                             under rotations and reflections)
%oType:          Constraints on the colours
%                   'none'   -> no constraints
%                   'strong' -> no two beads can have the same colour
%                               (strong ordering)
%pType:         The type of patterns to be considered. Can Be:
%                   'all'  -> all the patterns are considered.
%                   'uni'  -> only uniform patterns are considered. All the 
%                             non-uniform patterns are clustered in one bin.
%                             Uniform patterns are those having at most two
%                             transitions in the periphery (see [1]).
%**********
%OUTPUT
%invCodemap:    The codemap of codes invariant under the action of tGroup 
%invCodes:      All the possible codes invariant under the action of tGroup 
%**********
     
    invClasses = [];
    nonInvClasses = [];
    
    %Generate (or read) the look-up table of action-invariant codes
    cache = ['./GInvLut_' num2str(nBeads)...
                      '_' num2str(nColours)...
                      '_' tGroup ...
                      '_' oType ...
                      '_' nType ...
                      '_' pType ...
                      '.mat'];
    if exist(cache)
        lut = load(cache);
        nonInvClasses = lut.lut.nonInvClasses;
        invClasses = lut.lut.invClasses;
    else
      
        [~, nonInvClasses, invClasses] = Orbits(nBeads,nColours,...
                                         tGroup,oType,nType,pType);
            
        lut = struct('nonInvClasses', nonInvClasses,...
                     'invClasses', invClasses);
        save(cache, 'lut');
    end
            
    %Do the substitution: replace the non invariant codes
    %with the action-invariant ones
    [~,loc] = ismember(codemap,nonInvClasses);
    invCodemap = invClasses(loc);
    invCodes = unique(invClasses)';   
end