function [codemap,h] = HEP_POloe(img, r, ftrType, crcType, fltType)
%An implementation of Loewner order based descriptors using digital circles
%**********
%Synopsis:  [codemap,h] = HEP_POrgb(img, r, ftrType, crcType, fltType)
%**********
%INPUT
%img:       The input image. An n x m x 3 matrix of double or
%           integers in RGB format
%r:         The radius of the circle. An integer representing the resolution 
%           (in pixels) at which TS is computed.
%ftrType:   The feature type. Can be:
%               -> 'dir'    (directional, non rotation-invariant features)
%               -> 'CN'     (rotation-invariant features, i.e.: cyclic group CN)
%               -> 'DN'     (rotation and reflection-invariant features, 
%                            i.e.: dihedral group DN)
%crcType:   The type of digital circle - for possible values see DrawCircle()
%fltType:   How filtering is performed across the image borders - for
%           possible values see CreateImageLayers()
%
%**********
%OUTPUT
%codemap:   The map of POloe codes (codeword image)
%h:         The normalised histogram of POloe codes (sums one)
%**********
%
%   Sample usage
%   --------
%      %Example 1
%      %Resolution = 2 pixels, directional features, circular repetition across
%      %the image borders, Andres' circles
%      img = imread('rice.png');
%      [codemap, h] = HEP_POloe(img, 2, 'dir', 'Andres', 'circular');
%
%      %Example 2
%      %Resolution = 1 pixels, rotation-invariant features, symmetric repetition across
%      %the image borders, Andres' circles
%      img = imread('rice.png');
%      [codemap, h] = HEP_POloe(img, 2, 'ri', 'Andres', 'symmetric');

% Version published in 2016 by Francesco Bianconi 
% Department of Engineering, Universit degli Studi di Perugia, Italy
% Hacked by Fabrizio Smeraldi, 2016
% School of Electronic Engineering and Computer Science
% Queen Mary, University of London
    
    
    % Check we have a RGB image
    if size(img,3)~=3
        error('PO features require an RGB image');
    end

    % Convert it to HCLtilde
    hclimg=rgb2hcl(img);
    
    %Generate the neighbourhood
    disps = DrawCircle(r, crcType);
    
    % Layerize the input image moving the colour channels
    % to the fourth dimension
    Lh = CreateImageLayers(hclimg(:,:,1),disps,fltType);
    Lc = CreateImageLayers(hclimg(:,:,2),disps,fltType);
    Ll = CreateImageLayers(hclimg(:,:,3),disps,fltType);
    
    %Compute the TS codes
    mask = 3.^(0:size(Lh,3)-1);                  %Mask of weights
    codes = 0:(3^size(Lh,3))-1;                  %Possible codes
    mask = reshape(mask, [1 1 numel(mask)]);
    mask = repmat(mask, [size(Lh,1) size(Lh,2) 1]);
    I_copy_h = repmat(hclimg(:,:,1), [1 1 size(Lh,3)]);
    I_copy_c = repmat(hclimg(:,:,2), [1 1 size(Lh,3)]);
    I_copy_l = repmat(hclimg(:,:,3), [1 1 size(Lh,3)]);

    % Difference between image centre and peripheral points
    delta_h=I_copy_h-Lh;
    delta_c=I_copy_c-Lc;
    delta_l=I_copy_l-Ll;
    % Matrix elements
    alpha = (delta_l-delta_c)/sqrt(2);
    beta = delta_h/sqrt(2);
    gamma = (delta_l+delta_c)/sqrt(2);
    
    %Assign 0, 1 or 2 if the central pixel is lower than, greater equal to or
    %not comparable with the peripheral one
    comparisons=2*ones(size(alpha));
    % first condition for comparability: alpha * gamma >= beta^2
    ag_gt_b2= alpha.*gamma >= beta.*beta;
    % If the 1st condition is true and alpha>= and gamma>=0 then centre is
    % greater equal - take away one
    comparisons=comparisons - ag_gt_b2 .* ((alpha>=0) & ...
                                           (gamma>=0));
    % If the 1st condition is true and alpha<0 and gamma<0 then
    % centre is smaller - take away 2
    comparisons=comparisons - 2*ag_gt_b2 .* ((alpha<0) & ...
                                             (gamma<0));
    
    %Compute the codemap
    codemap = sum(comparisons.*mask,3);
    
    %Postprocess the codes if required
    if strcmp(ftrType,'dir')
        %Do nothing
    elseif strcmp(ftrType,'CN') || strcmp(ftrType,'DN') 
        nBeads = size(Lh,3);
        nColours = 3;
        nType = 'PERI';
        [codemap, codes] = PostprocessCodes(codemap, nBeads,...
            nColours, nType, ftrType);
    else
        error('Feature type unsupported');
    end
    
    %Compute the normalised histogram
    h = hist(codemap(:), codes)/numel(codemap);
end