function [codemap,h] = HEP_POrgb(img, r, ftrType, crcType, fltType)
%An implementation of Texture Spectrum based on digital circles
%**********
%Synopsis:  [codemap,h] = HEP_POrgb(img, r, ftrType, crcType, fltType)
%**********
%INPUT
%img:       The input image. An n x m x 3 matrix of double or
%           integers in RGB format
%r:         The radius of the circle. An integer representing the resolution 
%           (in pixels) at which TS is computed.
%ftrType:   The feature type. Can be:
%               -> 'dir'    (directional, non rotation-invariant features)
%               -> 'CN'     (rotation-invariant features, i.e.: cyclic group CN)
%               -> 'DN'     (rotation and reflection-invariant features, 
%                            i.e.: dihedral group DN)
%crcType:   The type of digital circle - for possible values see DrawCircle()
%fltType:   How filtering is performed across the image borders - for
%           possible values see CreateImageLayers()
%
%**********
%OUTPUT
%codemap:   The map of POrgb codes (codeword image)
%h:         The normalised histogram of POrgb codes (sums one)
%**********
%
%   Sample usage
%   --------
%      %Example 1
%      %Resolution = 2 pixels, directional features, circular repetition across
%      %the image borders, Andres' circles
%      img = imread('autumn.tif');
%      [codemap, h] = HEP_POrgb(img, 2, 'dir', 'Andres', 'circular');
%
%      %Example 2
%      %Resolution = 1 pixels, rotation-invariant features, symmetric repetition across
%      %the image borders, Andres' circles
%      img = imread('autumn.tif');
%      [codemap, h] = HEP_POrgb(img, 2, 'CN', 'Andres', 'symmetric');

% Version published in 2016 by Francesco Bianconi 
% Department of Engineering, Universit degli Studi di Perugia, Italy
% Hacked by Fabrizio Smeraldi, 2016
% School of Electronic Engineering and Computer Science
% Queen Mary, University of London
%
% Bug fix 16/6/2016: <= now computed correctly    
    
    % Check we have a RGB image
    if size(img,3)~=3
        error('PO features require an RGB image');
    end
        
    %Generate the neighbourhood
    disps = DrawCircle(r, crcType);
    
    % Layerize the input image moving the colour channels
    % to the fourth dimension
    Lr = CreateImageLayers(img(:,:,1),disps,fltType);
    Lg = CreateImageLayers(img(:,:,2),disps,fltType);
    Lb = CreateImageLayers(img(:,:,3),disps,fltType);
    % concatenate along fourth dimension
    L=cat(4, Lr, Lg, Lb);
    
    
    %Compute the TS codes
    mask = 3.^(0:size(L,3)-1);                  %Mask of weights
    codes = 0:(3^size(L,3))-1;                  %Possible codes
    mask = reshape(mask, [1 1 numel(mask)]);
    mask = repmat(mask, [size(L,1) size(L,2) 1]);
    I_copyr = repmat(img(:,:,1), [1 1 size(L,3)]);
    I_copyg = repmat(img(:,:,2), [1 1 size(L,3)]);
    I_copyb = repmat(img(:,:,3), [1 1 size(L,3)]);
    % concatenate along fourth dimension
    I_copy = cat(4, I_copyr, I_copyg, I_copyb);
    
    %Compare the central pixel with the peripheral one
    %Assign 0, 1 or 2 if the central pixel is lower than, greater equal to or
    %not comparable with the peripheral one
    larger= I_copy>=L;
    smaller= I_copy<=L;
    equal= larger & smaller;
    comparisons=-(larger(:,:,:,1) & larger(:,:,:,2) & ...
                  larger(:,:,:,3));
    comparisons=comparisons-2*...
        (smaller(:,:,:,1) & smaller(:,:,:,2) & smaller(:,:,:,3));
    comparisons=comparisons+2*...
        (equal(:,:,:,1) & equal(:,:,:,2) & equal(:,:,:,3));
    comparisons=comparisons+2*ones(size(comparisons));
    
    %Compute the codemap
    codemap = sum(comparisons.*mask,3);
    
    %Postprocess the codes if required
    if strcmp(ftrType,'dir')
        %Do nothing
    elseif strcmp(ftrType,'CN') || strcmp(ftrType,'DN') 
        nBeads = size(L,3);
        nColours = 3;
        nType = 'PERI';
        [codemap, codes] = PostprocessCodes(codemap, nBeads,...
            nColours, nType, ftrType);
    else
        error('Feature type unsupported');
    end
    
    %Compute the normalised histogram
    h = hist(codemap(:), codes)/numel(codemap);
end