function features = ComputeHEP(imgSource, method, res, fType, nType)
%Image features based on Histograms of Equivalent Patterns
%**********
%Synopsis
%**********
%INPUT:
%imgSource:        Full path to the directory containing the source images  
%method:           The image descriptor to be used. Can be:
%                       'BGC'        -> Binary Gradient Contour
%                       'CLBP_MC_S'  -> Complete Local Binary Patterns 
%                                       Variant: M x C || S
%                       'GBLBP'      -> Gradient-based Local Binary Patterns
%                       'ILBP'       -> Improved Local Binary patterns
%                       'LBP'        -> Local Binary Patterns
%                       'LTP'        -> Local Ternary Patterns
%                       'RT'         -> Rank Transform
%                       'TS'         -> Texture Spectrum 
%res:              The resolution at which the features are computed. This is 
%                  an array of integers representing the radii of the circular
%                  neighbourhoods used to compute the features. 
%                  If numel(res) > 1 the resulting features are the concatenation 
%                  of the features corresponding to each radius.
%fType:            The type of features to be computed. Can be:
%                       'dir'   -> Directional features
%                       'CN'    -> Rotation-invariant features, i.e.: features 
%                                  invariant under the cyclic group CN
%                       'DN'    -> Rotation and reflection-invariant features, 
%                                  i.e.: features invariant under the cyclic group CN
%nType:            The type of circular neigbourhood used to compute the
%                  features. Can be:
%                       'Andres'                -> Andres' digital circles
%                       'eight-point-square'    -> Eight-point square
%                                                  neighbourhood
%**********
%OUTPUT:
%features:         The feature vector. 
%**********
%
%   Sample usage
%   --------
%      %Example 1
%      %Compute directional Local Binary Patterns at resolution 2 using
%      %Andres' digital circle.
%      imgSource = './MyImages';
%      method = 'LBP';
%      res = 2;
%      fType = 'dir';
%      nType = 'Andres';
%      feats = ComputeHEP(imgSource, method, res, fType, nType)
%
%      %Example 2
%      %Compute rotation-invariant Texture Spectrum at resolutions 1 and 2
%      img = imread('rice.png');
%      [features] = HEP_TS(img, [1 2], 'dir', 'Andres', 'circular');
%
%Author:            Francesco Bianconi
%Last updated:      May 2016

    features = [];    
    fltType = 'circular';
    
    %Path to temporary image (used if conversion to grey-scale is needed)
    tempImgFile = ['./tempImg.bmp'];
    
    %Read the images' filenames
    images = GetFileNames(imgSource);
    
    for i = 1:size(images,1)
    
        imgFile = images(i,:);
        disp(['Computing--' method '--' num2str(res) '--' fType '--' imgFile]);
        
        %Read the image and the image's info
        [img, map] = imread(imgFile);
        imgInfo = imfinfo(imgFile);
        bitDepth = imgInfo.BitDepth;
        colourType = imgInfo.ColorType;

        %Convert the image to grey-scale if required
        if strcmp(colourType,'grayscale') 
            greyimg=img;
        elseif strcmp(colourType,'indexed')
            greyimg = ind2gray(img, map);
        elseif strcmp(colourType,'truecolor')
            greyimg = ConvertToGrey(img);
        else
            error('Unsupported image type');
        end

        %Compute the features for each resolution then concatenate them
        F = [];
        for r = 1:numel(res)
            switch method
                case 'BGC'
                    [~,f] = HEP_BGC(greyimg, res(r), fType, nType, fltType);
                case 'CLBP_MC_S'
                    [f] = HEP_CLBP_MC_S(greyimg, res(r), fType, nType, fltType);
                case 'GBLBP'
                    [~,f] = HEP_GBLBP(greyimg, res(r), fType, nType, fltType);
                case 'ILBP'
                    [~,f] = HEP_ILBP(greyimg, res(r), fType, nType, fltType);
                case 'LBP'
                    [~,f] = HEP_LBP(greyimg, res(r), fType, nType, fltType);
                case 'LTP'
                    [~,f] = HEP_LTP(greyimg, res(r), fType, nType, fltType);
                case 'RT'
                    [~,f] = HEP_RT(greyimg, res(r), nType, fltType);
                case 'TS'
                    [~,f] = HEP_TS(greyimg, res(r), fType, nType, fltType);
                case 'POrgb'
                    [cmap,f] = HEP_POrgb(img, res(r), fType, nType, fltType);
                case 'POloe'
                    [cmap,f] = HEP_POloe(img, res(r), fType, nType, fltType);
                otherwise
                    error('Unsupported descriptor');
            end
            F = [F, f];
        end 
        features = [features; F];
    end
end

function [I, imgFile] = ConvertToGrey(img)

%Converts a colour image into grey-scale
%
%INPUT
%img    :   input image
%
%
%OUTPUT
%I          :   grey-scale output image
%imgFile    :   path to the temporary image
    
    I = rgb2gray(img);
    
    imgFile = './tempImg.bmp';
    imwrite(I, imgFile, 'bmp');
end

