function [codemap,h] = HEP_ILBP(img, r, ftrType, crcType, fltType)
%An implementation of Improved Local Binary Patterns based on digital circles
%
%INPUT
%img:       The input image. An n x m matrix of double or integers.
%r:         The radius of the circle. An integer representing the resolution 
%           (in pixels) at which ILBP is computed.
%ftrType:   The feature type. Can be:
%               -> 'dir'    (directional, non rotation-invariant features)
%               -> 'ri'     (rotation-invariant features)
%crcType:   The type of digital circle - for possible values see DrawCircle()
%fltType:   How filtering is performed across the image borders - for
%           possible values see CreateImageLayers()
%
%OUTPUT
%codemap:   The map of ILBP codes (codeword image)
%h:         The normalised histogram of ILBP codes (sums one)
%
%
%   Sample usage
%   --------
%      %Example 1
%      %Resolution = 2 pixels, directional features, circular repetition across
%      %the image borders, Andres' circles
%      img = imread('rice.png');
%      [codemap, h] = HEP_ILBP(img, 2, 'dir', 'Andres', 'circular');
%
%      %Example 2
%      %Resolution = 1 pixels, rotation-invariant features, symmetric repetition across
%      %the image borders, Andres' circles
%      img = imread('rice.png');
%      [codemap, h] = HEP_ILBP(img, 2, 'ri', 'Andres', 'symmetric');

% Version published in 2016 by Francesco Bianconi 
% Department of Engineering, Universit degli Studi di Perugia, Italy

    %Generate the neighbourhood
    disps = DrawCircle(r, crcType);
    
    %Layerize the input image
    L = CreateImageLayers(img,disps,fltType);
    
    %Compute the ILBP codes
    mask = 2.^(0:size(L,3));  %Mask of weights
    mask = reshape(mask, [1 1 numel(mask)]);
    mask = repmat(mask, [size(L,1) size(L,2) 1]);
    average = mean(cat(3,L,img),3);
    average = repmat(average, [1 1 size(L,3) + 1]);
    codemap = sum((cat(3,L,img) <= average).*mask,3);
    
    %Postprocess the codes if required
    nFeats = 0;     %Number of features 
    switch ftrType
        case 'dir'
            nFeats = (2^(size(L,3)+1));
        case 'ri'
            %Generate the lookup table for rotation invariance 
            %(necklaces with n beads and k colours)
            n = size(L,3);
            k = 2;
            lut = NecklacesClassNumber(n, k);
            lut(:,1) = [];
            nFeats = numel(unique(lut));
            codemap = lut(codemap + 1);
        otherwise
            error('Unsupported feature type')
    end
    
    %Compute the normalised histogram
    h = hist(codemap(:), 0:nFeats-1)/numel(codemap);
    
    %First bin is 0 by definition
    h(1) = [];
end