function [codemap,h] = HEP_IOCLBP_Fieguth_channels(img, cSpaceIntra, rIntra, cSpaceInter, rInter, ftrType, crcType, fltType, IntraChannel, InterChannel, nL)
%Improved Opponent Color Local Binary Patterns
%**********
%Synopsis:  HEP_IOCLBP_Fieguth_channels(img, cSpaceIntra, rIntra, cSpaceInter, rInter, ftrType, crcType, fltType, IntraChannel, InterChannel, nL)
%**********
%INPUT
%img:               The input image. An N x M x 3 RGB image.
%cSpaceIntra:       The colour space used to compute the intra-channel features. 
%                   Can be:
%                       -> 'RGB'
%                       -> 'Ohta'
%                       -> 'Opponent'
%                       -> 'YIQ'
%                       -> 'YUV'
%                       -> 'HSV'
%                       -> 'HSVnorm'    [Hnorm = (H - mean(H))/std(H)]
%rIntra:            The radius/radii (in pixels) of the centre-symmetric 
%                   neighbourhood(s) used to compute the intra-channel features.
%                   This is a single integer value or an array of
%                   N integers. The corresponding features vectors are
%                   concatenated if N > 1.
%cSpaceInter:       Same as 'cSpaceIntra' but for inter-channel features. 
%rInter:            Same as 'rIntra' but for inter-channel features.
%ftrType:           The feature type. Can be:
%                       -> 'dir'    (directional, non rotation-invariant features)
%                       -> 'CN'     (rotation-invariant features, i.e.: cyclic group CN)
%                       -> 'DN'     (rotation and reflection-invariant features, 
%                                    i.e.: dihedral group DN)
%crcType:           The type of digital circle - for possible values see
%                   DrawCircle().
%fltType:           How filtering is performed across the image borders - for
%                   possible values see CreateImageLayers().
%IntraChannel:      The channels of the cSpaceIntra for which the features
%                   are calculated. Can be:
%                       'None'(only INTER-channel features)
%                       '1', '2', '3' (1st, 2nd or 3rd channel)
%                       '12', '13', '23' (1st+2nd, 1st+3rd or 2nd + 3rd channel)
%                       'All' (1st+2nd+3rd channel)
%InterChannel:      The channels of the cSpaceInter for which the features
%                   are calculated. Can be:
%                       'None'(only INTRA-channel features)
%                       '12', '13', '23' (1st/2nd, 1st/3rd or 2nd/3rd channel)
%                       '12+13', '12+23', '13+23' (1st/2nd + 1st3rd, etc.)
%                       'All' (1st/2nd + 1st/3rd + 2nd/3rd channel)
%nL:                Number of levels   which each channel is encoded
%                   (typical value is 256).
%**********
%OUTPUT
%codemap:           The maps of IOCLBP codes (codeword image s). An N x M x 6 matrix
%                   arranged as follows:
%                       Layer 1 -> intra-channel codes (C1)
%                       Layer 2 -> intra-channel codes (C2)
%                       Layer 3 -> intra-channel codes (C3)
%                       Layer 4 -> inter-channel codes (C1/C2)
%                       Layer 5 -> inter-channel codes (C1/C3)
%                       Layer 6 -> inter-channel codes (C2/C3)
%
%h:                 The normalised histogram (sums one).
%**********
%
%   Sample usage
%   --------
%   Example 1:
%
%       Resolution = [1 2 3] (1px, 2px and 3px concatenated), the same for INTRA- and INTER-channel features. 
%       Directional features. 
%       Circular repetition across the image borders. 
%       Andres' circles. 
%       RGB colour space for INTRA- and INTER-channel features.
%       All the channels considered for INTRA- and INTER-channel features.
%
%      img = imread('peppers.png');
%      [codemap, h] = HEP_IOCLBP_Fieguth_channels(img, 'RGB', [1 2 3], 'RGB', [1 2 3], 'dir', 'Andres', 'circular',...
%                                                 'All', 'All');
%
%   Example 2:
%
%       Resolution = 1 pixel for INTRA- and  [1 3] for INTER- (1 px and 3 px concatenated).
%       Rotation-invariant features.
%       Symmetric repetition across the image borders.
%       Eight-point circles.
%       HSV colour space for INTRA- and RGB for INTER-channel features.
%       All the channels considered for INTRA- and only 'Rg' for INTER-.
%
%       img = imread('peppers.png');
%       [codemap, h] = HEP_IOCLBP_Fieguth_channels(img, 'HSV', 1, 'RGB', [1 3], 'CN', 'eight-point-circle', 'symmetric',...
%                                                 'All', 'Rg', 256);
%
%
%   Example 3:
%       
%       Only INTER-channel features.
%       Resolution = 1 pixel.
%       Rotation-invariant features.
%       Circular repetition across the image borders.
%       Eight-point circles.
%       RGB colour space.
%       All the channels considered.
%
%       img = imread('peppers.png');
%       [codemap, h] = HEP_IOCLBP_Fieguth_channels(img, 'None', 0, 'RGB', 1, 'CN', 'eight-point-circle', 'circular',...
%                                                 'None', 'All');

% Version published in 2016 by Francesco Bianconi 
% Department of Engineering, Universit degli Studi di Perugia, Italy

    switch cSpaceIntra

        case {'None', 'RGB'}
            imgIntra = img;
        case 'Ohta'
            [imgIntra] = RGB_to_OHTA(img, nL);
        case 'Opponent'
            [imgIntra] = RGB_to_OPP(img, nL);
        case 'YIQ'
            [imgIntra] = RGB_to_YIQ(img, nL);
        case 'YUV'
            [imgIntra] = RGB_to_YUV(img, nL);
        case 'HSV'
            [imgIntra] = RGB_to_HSV(img, nL);
        case 'HSVnorm'
            [imgIntra] = RGB_to_HSVnorm(img, nL);
        otherwise
            error('Unsupported colour space');
    end
    
    switch cSpaceInter

        case {'None', 'RGB'}
            imgInter = img;
        case 'Ohta'
            [imgInter] = RGB_to_OHTA(img, nL);
        case 'Opponent'
            [imgInter] = RGB_to_OPP(img, nL);
        case 'YIQ'
            [imgInter] = RGB_to_YIQ(img, nL);
        case 'YUV'
            [imgInter] = RGB_to_YUV(img, nL);
        case 'HSV'
            [imgInter] = RGB_to_HSV(img, nL);
        case 'HSVnorm'
            [imgInter] = RGB_to_HSVnorm(img, nL);
        otherwise
            error('Unsupported colour space');
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% Separate the channels %%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    imgIntraC1 = imgIntra(:,:,1);
    imgIntraC2 = imgIntra(:,:,2);
    imgIntraC3 = imgIntra(:,:,3);

    imgInterC1 = imgInter(:,:,1);
    imgInterC2 = imgInter(:,:,2);
    imgInterC3 = imgInter(:,:,3);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% Compute the INTRA-channel codemaps and histograms %%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    codemapIntra = [];
    hIntra = [];
    for i = 1:numel(rIntra)
    [codemapC1, histC1] = HEP_ILBP(imgIntraC1, rIntra(i), ftrType, crcType, fltType);
    [codemapC2, histC2] = HEP_ILBP(imgIntraC2, rIntra(i), ftrType, crcType, fltType);
    [codemapC3, histC3] = HEP_ILBP(imgIntraC3, rIntra(i), ftrType, crcType, fltType);
    
        switch IntraChannel
            
        case 'None'
            nIntra = 0;
            codemapIntra = [];
            hIntra = [];
        case {'1'}
            nIntra = 1;
            codemapIntra = cat(3, codemapIntra, codemapC1);
            hIntra = [hIntra, histC1];
        case {'2'}
            nIntra = 1;
            codemapIntra = cat(3, codemapIntra, codemapC2);
            hIntra = [hIntra, histC2];
        case {'3'}
            nIntra = 1;
            codemapIntra = cat(3, codemapIntra, codemapC3);
            hIntra = [hIntra, histC3];
        case {'12'}
            nIntra = 2;
            codemapIntra = cat(3, codemapIntra, codemapC1, codemapC2);
            hIntra = [hIntra, histC1, histC2];  
        case {'13'}
            nIntra = 2;
            codemapIntra = cat(3, codemapIntra, codemapC1, codemapC3);
            hIntra = [hIntra, histC1, histC3];  
        case {'23'}
            nIntra = 2;
            codemapIntra = cat(3, codemapIntra, codemapC2, codemapC3);
            hIntra = [hIntra, histC2, histC3]; 
        case {'All'}
            nIntra = 3;
            codemapIntra = cat(3, codemapIntra, codemapC1, codemapC2, codemapC3);
            hIntra = [hIntra, histC1, histC2, histC3];

        otherwise
            error('Unsupported combination');
        end 
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% Compute the INTER-channel codemaps and histograms %%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    codemapInter = [];
    hInter = [];
    for j = 1:numel(rInter)
        
        %%% Generate the neighbourhood %%%
        disps = DrawCircle(rInter(j), crcType);
    
        %%% Layerize the input image %%%
        L1 = CreateImageLayers(imgInterC1,disps,fltType);
        L1 = cat(3, L1, imgInterC1);
        L2 = CreateImageLayers(imgInterC2,disps,fltType);
        L2 = cat(3, L2, imgInterC2);
        L3 = CreateImageLayers(imgInterC3,disps,fltType);
        L3 = cat(3, L3, imgInterC3);
        averageC1 = repmat(mean(L1,3), [1 1 size(disps,1) + 1]);
        averageC2 = repmat(mean(L2,3), [1 1 size(disps,1) + 1]);
        averageC3 = repmat(mean(L3,3), [1 1 size(disps,1) + 1]);
    
        %%% Define the weight mask %%%
        mask = 2.^(0:size(disps,1));                 
        codes = 0:sum(ones(size(mask)).*mask);     
        mask = reshape(mask, [1 1 numel(mask)]);
        mask = repmat(mask, [size(imgInter,1) size(imgInter,2) 1]);
    
        %%% Compute the INTER-channel codemaps C1/C2, C1/C3 and C2/C3 %%%   
        comparisonsC1C2 = (L1 <= averageC2);
        codemapC1C2 = sum(comparisonsC1C2.*mask,3);
        comparisonsC1C3 = (L1 <= averageC3);
        codemapC1C3 = sum(comparisonsC1C3.*mask,3);
        comparisonsC2C3 = (L2 <= averageC3);
        codemapC2C3 = sum(comparisonsC2C3.*mask,3);
    
        %%% Postprocess the codes if required %%%
        if strcmp(ftrType,'dir')
            %Do nothing
        elseif strcmp(ftrType,'CN') || strcmp(ftrType,'DN') 
            nBeads = size(disps,1) + 1;
            nColours = 2;
            nType = 'FULL';
            [codemapC1C2, codes] = PostprocessCodes(codemapC1C2, nBeads,...
                nColours, nType, ftrType);
            [codemapC1C3, codes] = PostprocessCodes(codemapC1C3, nBeads,...
                nColours, nType, ftrType);
            [codemapC2C3, codes] = PostprocessCodes(codemapC2C3, nBeads,...
                nColours, nType, ftrType);
        else
            error('Feature type unsupported');
        end
    
        %%% Compute and concatenate the normalised histogram %%%
        histC1C2 = hist(codemapC1C2(:), codes)/numel(codemapC1C2);
        histC1C3 = hist(codemapC1C3(:), codes)/numel(codemapC1C3);
        histC2C3 = hist(codemapC2C3(:), codes)/numel(codemapC2C3);
    
        switch InterChannel
            case 'None'
                nInter = 0;
                codemapInter = [];
                hInter = [];
            case {'12'}
                nInter = 1;
                codemapInter = cat(3, codemapInter, codemapC1C2);
                hInter = [hInter, histC1C2];
            case {'13'}
                nInter = 1;
                codemapInter = cat(3, codemapInter, codemapC1C3);
                hInter = [hInter, histC1C3];
            case {'23'}
                nInter = 1;
                codemapInter = cat(3, codemapInter, codemapC2C3);
                hInter = [hInter, histC2C3];
            case {'12+13'}
                nInter = 2;
                codemapInter = cat(3, codemapInter, codemapC1C2, codemapC1C3);
                hInter = [hInter, histC1C2, histC1C3]; 
            case {'13+23'}
                nInter = 2;
                codemapInter = cat(3, codemapInter, codemapC1C3, codemapC2C3);
                hInter = [hInter, histC1C3, histC2C3]; 
            case {'12+23'}
                nInter = 2;
                codemapInter = cat(3, codemapInter, codemapC1C2, codemapC2C3);
                hInter = [hInter, histC1C2, histC2C3]; 
            case 'All'
                nInter = 3;
                codemapInter = cat(3, codemapInter, codemapC1C2, codemapC1C3, codemapC2C3);
                hInter = [hInter, histC1C2, histC1C3, histC2C3];

            otherwise
                error('Unsupported combination');
            end 
    
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% Put everything together %%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    codemap = cat(3, codemapIntra, codemapInter);
    h = [hIntra, hInter];
    
end