function featureVector = IGLCMFeatures(img, d, nt, avg, fn, bitDepth)
%Integrative co-occurrence features
%**********
%Synopsis:      featureVector = IGLCMFeatures(img, d, nt, avg, fn, bitDepth)
%**********
%INPUT:
%img            A three-channel colour input image
%d              The distance in pixels at which the co-occurrence matrices 
%               are computed   
%nt             The type of neighbourhood. Can be:
%                   'HAR'  Haralick's original implementation, i.e.: 
%                          four directions at 0, 45, 90 and 135
%                          See Ref. [2].
%                   'PET'  Petrou's digital circles - see Ref. [3].
%avg            Whether the matrices should be averaged or not (true/false)
%fn             The type of normalization for obtaining rotation-invariant 
%               features. Can be
%                   'NONE'          -> no normalization (directional features) 
%                   'AVG'           -> average
%                   'AVG+RANGE'     -> average + range
%                   'AVG+RANGE+MAD' -> average + range + mean absolute deviation
%                   'DFT'           -> discrete Fourier transform
%bitDepth       The bit depth of the input image
%**********
%OUTPUT:
%featureVector  The feature vector
%**********
%
%   Sample usage
%   --------
%      %Example 1
%      %Distance = 2 pixels, neighbourhood type = digital circle,
%      %directional features
%      img = imread('fabric.png');
%      bitDepth = 8;
%      f = IGLCMFeatures(img, 1, 'PET', false, 'NONE', bitDepth);
%
%
%References
%[1] V. Arvis, C. Debain, M. Berducat and A. Benassi; "Generalization of 
%    the cooccurrence matrix for colour image analysis"; Image Analysis and
%    Stereology, 23(1)
%[2] F. Bianconi and A. Fernndez, A.; "Rotation invariant co-occurrence 
%    features based on digital circles and discrete Fourier transform"
%    Pattern Recognition Letters, 48, pp. 34-41.
%[3] R. M. Haralick et al.; "Textural features for image classification." 
%    IEEE Transactions on Systems, Man, and Cybernetics, 3(6):610621, 1973.
%[4] M. Petrou and P. Garca Sevilla; "Image Processing. 
%    Dealing with Texture." Wiley Interscience, 2006.

    
    %Compute the intra-channel features
    CM = cat(3, COOCMatrices(img(:,:,1), d, nt, avg, bitDepth),...
                COOCMatrices(img(:,:,2), d, nt, avg, bitDepth),...
                COOCMatrices(img(:,:,3), d, nt, avg, bitDepth));

    %Compute the inter-channel features
    CM = cat(3, CM,...
                CrossCOOCMatrices(img(:,:,1), img(:,:,2), d, nt, avg, bitDepth),...
                CrossCOOCMatrices(img(:,:,1), img(:,:,3), d, nt, avg, bitDepth),...
                CrossCOOCMatrices(img(:,:,2), img(:,:,3), d, nt, avg, bitDepth));
    
    %Extract features
    featureVector = [];
    featureMatrix = [];
    for d = 1:size(CM,3)
        featureVector_ = [Contrast(CM(:,:,d)),... 
                          Correlation(CM(:,:,d)),... 
                          Energy(CM(:,:,d)),...
                          Entropy(CM(:,:,d)),...
                          Homogeneity(CM(:,:,d))];
        featureMatrix = [featureMatrix; featureVector_];
    end
    
    switch fn
        case 'NONE'
            %Do nothing
            featureVector = reshape(featureMatrix, 1, numel(featureMatrix));
        case 'AVG'
            featureVector = mean(featureMatrix);
        case 'AVG+RANGE'
            featureVector = [mean(featureMatrix), max(featureMatrix) - min(featureMatrix)];
        case 'AVG+MAD'
            featureVector = [mean(featureMatrix), mad(featureMatrix)];
        case 'AVG+RANGE+MAD'
            featureVector = [mean(featureMatrix),...
                             max(featureMatrix) - min(featureMatrix),...
                             mad(featureMatrix)];
        case 'DFT'
            nDisp_redu = floor(nDisp/2) + 1;
            featureMatrix = featureMatrix/nDisp;
            featureMatrix = abs(fft(featureMatrix));
            featureVector = reshape(featureMatrix, 1, numel(featureMatrix));
        case 'DFT+RANGE'
            nDisp_redu = floor(nDisp/2) + 1; 
            featureMatrix = featureMatrix/nDisp;
            featureMatrix = abs(fft(featureMatrix));
            featureMatrix = featureMatrix(1:nDisp_redu,:);
            featureVector = reshape(featureMatrix, 1, numel(featureMatrix));
            featureVector = [featureVector, max(featureMatrix) - min(featureMatrix)];            
        otherwise error('Unsupported method for feature normalization');
    end
end

function [CN] = Contrast(CM)

    %Computes contrast of a co-occurrence matrix
    %
    %INPUT:
    %CM: the co-occurrence matrix
    %
    %OUTPUT
    %CN: contrast value (normalized between 0 and 1)
    %
    
    [rows cols] = size(CM);
    if (rows ~= cols)
        error('Co-occurrence matrix should be square');
    end 
    
    stats = graycoprops_modif(CM, 'Contrast');    
    CN = (1/((rows-1)^2))*(stats.Contrast);

end

function [CR] = Correlation(CM)

    %Computes correlation of a co-occurrence matrix
    %
    %INPUT:
    %CM: the co-occurrence matrix
    %
    %OUTPUT
    %CR: correlation value (normalized between 0 and 1)
    %
    
    [rows cols] = size(CM);
    if (rows ~= cols)
        error('Co-occurrence matrix should be square');
    end 
    
    stats = graycoprops_modif(CM, 'Correlation');
    CR = stats.Correlation;
    
    %Normalize 
    CR = (1/2)*CR + 1/2;
end

function [EN] = Energy(CM)

    %Computes energy of a co-occurrence matrix
    %
    %INPUT:
    %CM: the co-occurrence matrix
    %
    %OUTPUT
    %EN: energy (normalized between 0 and 1)
    %
    
    [rows cols] = size(CM);
    if (rows ~= cols)
        error('Co-occurrence matrix should be square');
    end 
    
    stats = graycoprops_modif(CM, 'Energy');    
    EN = stats.Energy;

end

function [ET] = Entropy(CM)

    %Computes entropy of a co-occurrence matrix
    %
    %INPUT:
    %CM: the co-occurrence matrix
    %
    %OUTPUT
    %ET: entropy (normalized between 0 and 1)
    %
    
    [rows cols] = size(CM);
    if (rows ~= cols)
        error('Co-occurrence matrix should be square');
    end 
    
    ET = 0;
    
    %Compute ENTROPY
    for r = 1:rows
        for c = 1:cols
            if CM(r,c) ~= 0
                ET = ET + CM(r,c)*log2((CM(r,c)));
            end
        end
    end

    %Normalize entropy (entropy is max when all the entries of the matrix are equally probable)
    ET = (-1/(2*log2(rows)))*ET;

end

function [H] = Homogeneity(CM)

    %Computes homogeneity of a co-occurrence matrix
    %
    %INPUT:
    %CM: the co-occurrence matrix
    %
    %OUTPUT
    %H: homogeneity value (normalized between 0 and 1)
    %
    
    [rows cols] = size(CM);
    if (rows ~= cols)
        error('Co-occurrence matrix should be square');
    end 
    
    stats = graycoprops_modif(CM, 'Homogeneity');    
    H = stats.Homogeneity;
    
end


