function [LUT] = LutNosaka(nPixelsPeriphery, neighbourhoodType)
%
%INPUT
%nPixelsPeriphery:      number of pixels in the periphery of the neighbourhood
%neighbourhoodType:     type of neighbourhood. Can be:
%                           'PERI' -> periphery only (no central pixel)
%                           'FULL' -> periphery + central pixels
%                       Convention about the pixel mask: the weight of the
%                       central pixel should be 2^(nPixelsPeriphery)
%                       
%
%OUTPUT
%LUT:                   the look-up table for invariant pairs of patterns. Cell
%                       array of dimension N^2 x nPixelsPeriphery, where N
%                       is the total number of patterns.

    LUT = cell(0);

    wb = waitbar(0,'Generating LUT...');
    
    for i = 0:(2^nPixelsPeriphery) - 1
        for j = 0:(2^nPixelsPeriphery) - 1
        
            %Compute binary patterns
            pattern_i = dec2bin(i, nPixelsPeriphery);
            pattern_j = dec2bin(j, nPixelsPeriphery);
               
            if strcmp(neighbourhoodType, 'PERI')
                
                %Compute equivalent pairs. Each discrete rotation generates
                %one. There are nPixelsPeriphery equivalent pairs.
                eqv_pairs  = cell(0);
                for k = 1:(nPixelsPeriphery)
                
                    eqv_pair = [bin2dec(circshift(pattern_i, [1 k])) ...
                                bin2dec(circshift(pattern_j, [1 k]))];
                    eqv_pairs = horzcat(eqv_pairs, eqv_pair);
                    
                end
            else
                %Case 'FULL'. Parameter 'c' is the (binary) value of the
                %central pixel.
                eqv_pairs_0  = cell(0);
                eqv_pairs_1  = cell(0);

                %Central pixel = 0
                c = 0;
                for k = 1:(nPixelsPeriphery)                
                    eqv_pair = [bin2dec([circshift(pattern_i, [1 k]) num2str(c)]) ...
                                bin2dec([circshift(pattern_j, [1 k]) num2str(c)])];
                    eqv_pairs_0 = horzcat(eqv_pairs_0, eqv_pair);
                end
                
                %Central pixel = 1
                c = 1;
                for k = 1:(nPixelsPeriphery)                
                    eqv_pair = [bin2dec([circshift(pattern_i, [1 k]) num2str(c)]) ...
                                bin2dec([circshift(pattern_j, [1 k]) num2str(c)])];
                    eqv_pairs_1 = horzcat(eqv_pairs_1, eqv_pair);
                end
                
                eqv_pairs = {eqv_pairs_0; eqv_pairs_1};
            end
            
            LUT = vertcat(LUT, eqv_pairs);            
        end
        
        waitbar((i + 1)*(j + 1)/(2^nPixelsPeriphery * 2^nPixelsPeriphery));
    end
    
    close(wb);
end